@@ -1,4 +1,7 @@
 /*  monkey-sound
+ *
+ *  arch-tag: Implementation of MP3 metadata loading
+ *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
  *                     Marco Pesenti Gritti <marco@it.gnome.org>
  *                     Bastien Nocera <hadess@hadess.net>
@@ -18,17 +21,16 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: mp3-stream-info-impl.c,v 1.3 2003/08/18 20:25:59 cwalters Exp $
  */
 
 #include <libgnomevfs/gnome-vfs.h>
-#include <libgnomevfs/gnome-vfs-mime-utils.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include <string.h>
 #include <stdlib.h>
 
 #include <id3tag.h>
 
+#include "rb-string-helpers.h"
+
 #include "id3-vfs/id3-vfs.h"
 
 #include "monkey-media-stream-info.h"
@@ -40,6 +42,7 @@ static void MP3_stream_info_impl_class_init (MP3StreamInfoImplClass *klass);
 static void MP3_stream_info_impl_init (MP3StreamInfoImpl *ma);
 static void MP3_stream_info_impl_finalize (GObject *object);
 static void MP3_stream_info_impl_open_stream (MonkeyMediaStreamInfo *info);
+static gboolean MP3_stream_info_impl_get_bitrate_info (MP3StreamInfoImpl *impl);
 static gboolean MP3_stream_info_impl_get_value (MonkeyMediaStreamInfo *info,
 					        MonkeyMediaStreamInfoField field,
 					        int index,
@@ -53,10 +56,21 @@ static char *MP3_stream_info_impl_id3_tag_get_utf8 (struct id3_tag *tag,
 static int MP3_stream_info_impl_get_n_values (MonkeyMediaStreamInfo *info,
 				              MonkeyMediaStreamInfoField field);
 
+struct MP3BitrateInfo
+{
+	int bitrate;
+	int samplerate;
+	int time;
+	int channels;
+	int version;
+	int vbr;
+};
+
 struct MP3StreamInfoImplPrivate
 {
 	struct id3_tag *tag;
 	struct id3_vfs_file *file;
+	struct MP3BitrateInfo *info_num;
 };
 
 static GObjectClass *parent_class = NULL;
@@ -125,7 +139,8 @@ MP3_stream_info_impl_finalize (GObject *object)
 
 	if (impl->priv->file != NULL)
 		id3_vfs_close (impl->priv->file);
-	
+
+	g_free (impl->priv->info_num);
 	g_free (impl->priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -155,6 +170,82 @@ MP3_stream_info_impl_open_stream (MonkeyMediaStreamInfo *info)
 	}
 
 	impl->priv->tag = id3_vfs_tag (impl->priv->file);
+
+	if (MP3_stream_info_impl_get_bitrate_info (impl) == FALSE)
+	{
+		error = g_error_new (MONKEY_MEDIA_STREAM_INFO_ERROR,
+				     MONKEY_MEDIA_STREAM_INFO_ERROR_OPEN_FAILED,
+				     _("Failed to gather information about the file"));
+		g_object_set (G_OBJECT (info), "error", error, NULL);
+		return;
+	}
+}
+
+static void
+MP3_stream_info_impl_get_length_from_tag (MP3StreamInfoImpl *impl)
+{
+	struct id3_frame const *frame;
+
+	/* The following is based on information from the
+	 * ID3 tag version 2.4.0 Native Frames informal standard.
+	 */
+	frame = id3_tag_findframe(impl->priv->tag, "TLEN", 0);
+
+	if (frame == NULL)
+		return;
+
+	{
+		union id3_field const *field;
+		unsigned int nstrings;
+		id3_latin1_t *latin1;
+
+		field = id3_frame_field (frame, 1);
+		nstrings = id3_field_getnstrings (field);
+		if (nstrings <= 0)
+			return;
+
+		latin1 = id3_ucs4_latin1duplicate
+			(id3_field_getstrings(field, 0));
+
+		if (latin1 == NULL)
+			return;
+
+		/* "The 'Length' frame contains the length of the
+		 * audio file in milliseconds, represented as a
+		 * numeric string."
+		 */
+		if (atol(latin1) > 0)
+			impl->priv->info_num->time = atol(latin1);
+		g_free (latin1);
+	}
+}
+
+static gboolean
+MP3_stream_info_impl_get_bitrate_info (MP3StreamInfoImpl *impl)
+{
+	if (impl->priv->info_num == NULL)
+	{
+		struct MP3BitrateInfo *info;
+
+		info = g_new0 (struct MP3BitrateInfo, 1);
+		if (id3_vfs_bitrate (impl->priv->file,
+				&info->bitrate,
+				&info->samplerate,
+				&info->time,
+				&info->version,
+				&info->vbr,
+				&info->channels) == 0)
+		{
+			impl->priv->info_num = info;
+			return FALSE;
+		}
+
+		impl->priv->info_num = info;
+
+		MP3_stream_info_impl_get_length_from_tag (impl);
+	}
+
+	return TRUE;
 }
 
 static int
@@ -435,13 +526,13 @@ MP3_stream_info_impl_get_value (MonkeyMediaStreamInfo *info,
 		}
 		break;
 	case MONKEY_MEDIA_STREAM_INFO_FIELD_DURATION:
+		g_value_init (value, G_TYPE_LONG);
+
+		if (impl->priv->info_num->vbr == 0)
 		{
-			int bitrate;
 			GnomeVFSFileSize size;
 			GValue val = { 0, };
 
-			g_value_init (value, G_TYPE_LONG);
-
 			MP3_stream_info_impl_get_value (info,
 							MONKEY_MEDIA_STREAM_INFO_FIELD_FILE_SIZE,
 							0,
@@ -449,12 +540,12 @@ MP3_stream_info_impl_get_value (MonkeyMediaStreamInfo *info,
 			size = g_value_get_long (&val);
 			g_value_unset (&val);
 
-			bitrate = id3_vfs_bitrate (impl->priv->file);
-
-			if (bitrate > 0)
-				g_value_set_long (value, ((double) size / 1024.0f) / ((double) bitrate / 8.0f));
+			if (impl->priv->info_num->bitrate > 0)
+				g_value_set_long (value, ((double) size / 1000.0f) / ((double) impl->priv->info_num->bitrate / 8.0f / 1000.0f));
 			else
 				g_value_set_long (value, 0);
+		} else {
+			g_value_set_long (value, impl->priv->info_num->time);
 		}
 		break;
 
@@ -464,22 +555,24 @@ MP3_stream_info_impl_get_value (MonkeyMediaStreamInfo *info,
 		g_value_set_boolean (value, TRUE);
 		break;
 	case MONKEY_MEDIA_STREAM_INFO_FIELD_AUDIO_CODEC_INFO:
+		g_value_init (value, G_TYPE_STRING);
+		if (impl->priv->info_num->version != 3)
 		{
-			int version = id3_vfs_version (impl->priv->file);
-			tmp = g_strdup_printf (_("MPEG %d Layer III"), version);
-			g_value_init (value, G_TYPE_STRING);
-			g_value_set_string (value, tmp);
-			g_free (tmp);
+			tmp = g_strdup_printf (_("MPEG %d Layer III"), impl->priv->info_num->version);
+		} else {
+			tmp = g_strdup (_("MPEG 2.5 Layer III"));
 		}
+		g_value_set_string (value, tmp);
+		g_free (tmp);
 		break;
 	case MONKEY_MEDIA_STREAM_INFO_FIELD_AUDIO_BIT_RATE:
 	case MONKEY_MEDIA_STREAM_INFO_FIELD_AUDIO_AVERAGE_BIT_RATE:
 		g_value_init (value, G_TYPE_INT);
-		g_value_set_int (value, id3_vfs_bitrate (impl->priv->file));
+		g_value_set_int (value, impl->priv->info_num->bitrate / 1000);
 		break;
 	case MONKEY_MEDIA_STREAM_INFO_FIELD_AUDIO_QUALITY:
 		g_value_init (value, MONKEY_MEDIA_TYPE_AUDIO_QUALITY);
-		g_value_set_enum (value, monkey_media_audio_quality_from_bit_rate (id3_vfs_bitrate (impl->priv->file)));
+		g_value_set_enum (value, monkey_media_audio_quality_from_bit_rate (impl->priv->info_num->bitrate / 1000));
 		break;
 	case MONKEY_MEDIA_STREAM_INFO_FIELD_AUDIO_TRM_ID:
 		/* FIXME */
@@ -488,15 +581,15 @@ MP3_stream_info_impl_get_value (MonkeyMediaStreamInfo *info,
 		break;
 	case MONKEY_MEDIA_STREAM_INFO_FIELD_AUDIO_VARIABLE_BIT_RATE:
 		g_value_init (value, G_TYPE_BOOLEAN);
-		g_value_set_boolean (value, id3_vfs_vbr (impl->priv->file));
+		g_value_set_boolean (value, impl->priv->info_num->vbr);
 		break;
 	case MONKEY_MEDIA_STREAM_INFO_FIELD_AUDIO_SAMPLE_RATE:
 		g_value_init (value, G_TYPE_LONG);
-		g_value_set_long (value, id3_vfs_samplerate (impl->priv->file));
+		g_value_set_long (value, impl->priv->info_num->samplerate);
 		break;
 	case MONKEY_MEDIA_STREAM_INFO_FIELD_AUDIO_CHANNELS:
 		g_value_init (value, G_TYPE_INT);
-		g_value_set_int (value, id3_vfs_channels (impl->priv->file));
+		g_value_set_int (value, impl->priv->info_num->channels);
 		break;
 	case MONKEY_MEDIA_STREAM_INFO_FIELD_AUDIO_SERIAL_NUMBER:
 		g_value_init (value, G_TYPE_LONG);
@@ -553,24 +646,27 @@ MP3_stream_info_impl_id3_tag_get_utf8 (struct id3_tag *tag, const char *field_na
 	if (frame == 0)
 		return NULL;
 
-	field = &frame->fields[1];
+	field = id3_frame_field(frame, 1);
 	nstrings = id3_field_getnstrings (field);
 	for (j = 0; j < nstrings; j++)
 	{
+		id3_utf8_t *tmp = NULL;
+
 		ucs4 = id3_field_getstrings (field, j);
 
 		if (strcmp (field_name, ID3_FRAME_GENRE) == 0)
 			ucs4 = id3_genre_name (ucs4);
 
-		utf8 = id3_ucs4_utf8duplicate (ucs4);
+		tmp = id3_ucs4_utf8duplicate (ucs4);
+		if (strcmp (tmp, "") != 0)
+			utf8 = tmp;
+		else
+			g_free (tmp);
 	}
 
 	if (utf8 && !g_utf8_validate (utf8, -1, NULL)) {
-		size_t read, written;
-
 		g_warning ("Invalid UTF-8 in %s field in mp3 file\n", field_name);
-
-		utf8 = g_locale_to_utf8 ((char *) ucs4, -1, &read, &written, NULL);
+		utf8 = rb_unicodify ((char *) ucs4);
 	}
 
 	return utf8;
