@@ -73,10 +73,6 @@ static void rb_iradio_source_get_property (GObject *object,
 static void rb_iradio_source_songs_show_popup_cb (RBEntryView *view,
 						  RhythmDBEntry *entry,
 						  RBIRadioSource *source);
-static void entry_added_cb (RBEntryView *view, RhythmDBEntry *entry,
-			    RBPropertyView *propview);
-static void entry_deleted_cb (RBEntryView *view, RhythmDBEntry *entry,
-			      RBPropertyView *propview);
 static void paned_size_allocate_cb (GtkWidget *widget,
 				    GtkAllocation *allocation,
 		                    RBIRadioSource *source);
@@ -93,6 +89,7 @@ static void rb_iradio_source_show_browser (RBIRadioSource *source,
 static void rb_iradio_source_state_prefs_sync (RBIRadioSource *source);
 static void genre_selected_cb (RBPropertyView *propview, const char *name,
 			       RBIRadioSource *iradio_source);
+static void rb_iradio_source_songs_view_sort_order_changed_cb (RBEntryView *view, RBIRadioSource *source);
 
 /* source methods */
 static const char *impl_get_status (RBSource *source);
@@ -150,7 +147,6 @@ struct RBIRadioSourcePrivate
 	gboolean loading_prefs;
 
 	RBIRadioQueryType query_type;
-	guint genre_add_handler_id;
 	char *search_text;
 	char *selected_genre;
 
@@ -310,6 +306,10 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_RATING);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_PLAY_COUNT);
 	rb_entry_view_append_column (source->priv->stations, RB_ENTRY_VIEW_COL_LAST_PLAYED);
+	g_signal_connect (G_OBJECT (source->priv->stations),
+			  "sort-order-changed",
+			  G_CALLBACK (rb_iradio_source_songs_view_sort_order_changed_cb),
+			  source);
 
 	g_signal_connect (G_OBJECT (source->priv->stations),
 			  "size_allocate",
@@ -326,13 +326,6 @@ rb_iradio_source_constructor (GType type, guint n_construct_properties,
 			  "property-selected",
 			  G_CALLBACK (genre_selected_cb),
 			  source);
-	source->priv->genre_add_handler_id =
-		g_signal_connect (G_OBJECT (source->priv->stations),
-				  "entry-added", G_CALLBACK (entry_added_cb),
-				  source->priv->genres);
-	g_signal_connect (G_OBJECT (source->priv->stations),
-			  "entry-deleted", G_CALLBACK (entry_deleted_cb),
-			  source->priv->genres);
 
 	g_object_set (G_OBJECT (source->priv->genres), "vscrollbar_policy",
 		      GTK_POLICY_AUTOMATIC, NULL);
@@ -417,13 +410,17 @@ rb_iradio_source_add_station (RBIRadioSource *source,
 
 	rhythmdb_write_lock (source->priv->db);
 
-	entry = rhythmdb_entry_new (source->priv->db, RHYTHMDB_ENTRY_TYPE_IRADIO_STATION, uri);
+	entry = rhythmdb_entry_lookup_by_location (source->priv->db, uri);
+	if (!entry)
+		entry = rhythmdb_entry_new (source->priv->db, RHYTHMDB_ENTRY_TYPE_IRADIO_STATION, uri);
 	g_value_init (&val, G_TYPE_STRING);
 	g_value_set_string (&val, title);
 	rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_TITLE, &val);
-	g_value_reset (&val);
-	g_value_set_string (&val, genre);
-	rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_GENRE, &val);
+	if (genre) {
+		g_value_reset (&val);
+		g_value_set_string (&val, genre);
+		rhythmdb_entry_set (source->priv->db, entry, RHYTHMDB_PROP_GENRE, &val);
+	}
 	g_value_unset (&val);
 		
 	rhythmdb_write_unlock (source->priv->db);
@@ -441,8 +438,20 @@ static void
 impl_search (RBSource *asource, const char *search_text)
 {
 	RBIRadioSource *source = RB_IRADIO_SOURCE (asource);
+	if (search_text == NULL && source->priv->search_text == NULL)
+		return;
+	if (search_text != NULL &&
+	    source->priv->search_text != NULL
+	    && !strcmp (search_text, source->priv->search_text))
+		return;
+	if (search_text != NULL && search_text[0] == '\0')
+		search_text = NULL;
+
 	g_free (source->priv->search_text);
-	source->priv->search_text = g_utf8_casefold (search_text, -1);
+	if (search_text)
+		source->priv->search_text = g_utf8_casefold (search_text, -1);
+	else
+		source->priv->search_text = NULL;
 	rb_iradio_source_do_query (source, RB_IRADIO_QUERY_TYPE_SEARCH);
 }
 
@@ -518,9 +527,9 @@ impl_get_status (RBSource *asource)
 {
  	RBIRadioSource *source = RB_IRADIO_SOURCE (asource);
 	char *ret;
-	ret = g_strdup_printf (_("%d total stations in %d distinct genres"),
-			       rb_entry_view_get_num_entries (source->priv->stations),
-			       rb_property_view_get_num_properties (source->priv->genres));
+	guint num_entries = rb_entry_view_get_num_entries (source->priv->stations);
+	ret = g_strdup_printf (ngettext ("%d station", "%d stations", num_entries),
+			       num_entries);
 	return ret;
 }
 
@@ -587,6 +596,13 @@ rb_iradio_source_state_pref_changed (GConfClient *client,
 	rb_iradio_source_state_prefs_sync (source);
 }
 
+static void
+rb_iradio_source_songs_view_sort_order_changed_cb (RBEntryView *view,
+						   RBIRadioSource *source)
+{
+	rb_debug ("sort order changed");
+	rb_iradio_source_do_query (source, RB_IRADIO_QUERY_TYPE_SEARCH);
+}
 
 static void
 rb_iradio_source_songs_show_popup_cb (RBEntryView *view,
@@ -637,26 +653,12 @@ rb_iradio_source_show_browser (RBIRadioSource *source,
 	}
 }
 
-static void
-entry_added_cb (RBEntryView *view, RhythmDBEntry *entry,
-		RBPropertyView *propview)
-{
-	rb_property_view_handle_entry_addition (propview, entry);
-}
-
-static void
-entry_deleted_cb (RBEntryView *view, RhythmDBEntry *entry,
-		  RBPropertyView *propview)
-{
-	rb_property_view_handle_entry_deletion (propview, entry);
-}
-
 static void
 rb_iradio_source_do_query (RBIRadioSource *source, RBIRadioQueryType qtype)
 {
 	RhythmDBQueryModel *query_model;
+	RhythmDBPropertyModel *genre_model;
 	GtkTreeModel *model;
-	GPtrArray *genre_query;
 	GPtrArray *query;
 
 	rhythmdb_read_lock (source->priv->db);
@@ -684,18 +686,20 @@ rb_iradio_source_do_query (RBIRadioSource *source, RBIRadioQueryType qtype)
 				       RHYTHMDB_QUERY_END);
 	}
 
-	g_signal_handler_block (G_OBJECT (source->priv->stations),
-				source->priv->genre_add_handler_id);
+	query_model = rhythmdb_query_model_new_empty (source->priv->db);
 
 	if (qtype < RB_IRADIO_QUERY_TYPE_GENRE) {
 		rb_property_view_reset (source->priv->genres);
 		g_free (source->priv->selected_genre);
 		source->priv->selected_genre = NULL;
-		g_signal_handler_unblock (G_OBJECT (source->priv->stations),
-					  source->priv->genre_add_handler_id);
-	}
+	} 
 
-	genre_query = rhythmdb_query_copy (query);
+	genre_model = rb_property_view_get_model (source->priv->genres);
+
+	if (qtype < RB_IRADIO_QUERY_TYPE_GENRE) {
+		g_object_set (G_OBJECT (genre_model), "query-model", query_model, NULL);
+	} else
+		g_object_set (G_OBJECT (genre_model), "query-model", NULL, NULL);
 
 	if (source->priv->selected_genre)
 		rhythmdb_query_append (source->priv->db,
@@ -705,21 +709,18 @@ rb_iradio_source_do_query (RBIRadioSource *source, RBIRadioQueryType qtype)
 				       source->priv->selected_genre,
 				       RHYTHMDB_QUERY_END);
 
-	query_model = rhythmdb_query_model_new_empty (source->priv->db);
-
 	model = GTK_TREE_MODEL (query_model);
+	
+	rb_entry_view_set_model (source->priv->stations, RHYTHMDB_MODEL (query_model));
 
 	rhythmdb_do_full_query_parsed (source->priv->db, model, query);
-	if (qtype >= RB_IRADIO_QUERY_TYPE_GENRE)
-		g_signal_handler_unblock (G_OBJECT (source->priv->stations),
-					  source->priv->genre_add_handler_id);
 
-	rhythmdb_query_free (genre_query);
+	if (qtype >= RB_IRADIO_QUERY_TYPE_GENRE)
+		g_object_set (G_OBJECT (genre_model), "query-model", query_model, NULL);
+
 	rhythmdb_query_free (query);
 
 	rhythmdb_read_unlock (source->priv->db);
-	
-	rb_entry_view_set_model (source->priv->stations, RHYTHMDB_MODEL (query_model));
 }
 
 static void
